-- actor2D.lua :: the implementation of a 2D actor. It contain every element
-- needed to create your own 2D actors.

--[[
  Copyright © 2019 Kazhnuz

  Permission is hereby granted, free of charge, to any person obtaining a copy of
  this software and associated documentation files (the "Software"), to deal in
  the Software without restriction, including without limitation the rights to
  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  the Software, and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local Actor2D = Object:extend()

-- INIT FUNCTIONS
-- Initialise the actor and its base functions

function Actor2D:new(world, type, x, y, w, h)
  self.type   = type or ""

  self:setManagers(world)
  self:initPhysics(x, y, w, h)

  self:initKeys()
  self:register()
end

function Actor2D:setManagers(world)
  self.world  = world
  self.scene  = world.scene
  self.obj    = world.obj
end

function Actor2D:initPhysics(x, y, w, h)
  self.x      = x or 0
  self.y      = y or 0
  self.w      = w or 0
  self.h      = h or 0

  self.xsp    = 0
  self.ysp    = 0
end

function Actor2D:register()
  self.world:registerActor(self)
end

-- INPUT FUNCTIONS
-- get input from the world object

function Actor2D:initKeys()
  self.keys = core.input.fakekeys
end

function Actor2D:getInput(keys)
  self.keys = keys or core.input.fakekeys
end

-- UPDATE FUNCTIONS
-- Theses functions are activated every steps

function Actor2D:update(dt)

  self:autoMove()
end

-- MOVEMENT FUNCTIONS
-- Basic functions from the movement.

function Actor2D:autoMove()
  self:move(self.x + self.xsp, self.y + self.ysp)
end

function Actor2D:move(newx, newy)
  self.world:moveActor(self, newx, newy)
end

-- DRAW FUNCTIONS
-- Draw the actors.

function Actor2D:draw()
  -- here will be update actions
end

return Actor2D
